@@ -1,8 +1,7 @@
 # -*- Perl -*-
 #
 # subpatterns are named A-Z and offer short-hand notation for e.g. V =>
-# a/e/i/o/u (or whatever) for vowels. these patterns can be changed
-# without a reparse of the pattern
+# a/e/i/o/u (or instead a reference to some other parse tree)
 
 package Lingua::Awkwords::Subpattern;
 
@@ -12,7 +11,7 @@ use Carp qw(confess croak);
 use Moo;
 use namespace::clean;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 # these defaults set from what the online version does at
 # http://akana.conlang.org/tools/awkwords/
@@ -25,29 +24,30 @@ my %patterns = (
 );
 
 has pattern => (
-    is  => 'rw',
-    isa => sub {
-        die "subpattern $_[0] does not exist" unless exists $patterns{ $_[0] };
+    is      => 'rw',
+    trigger => sub {
+        my ( $self, $pat ) = @_;
+        die "subpattern $pat does not exist" unless exists $patterns{$pat};
+        $self->_set_target( $patterns{$pat} );
     },
 );
+has target => ( is => 'rwp', );
 
 ########################################################################
 #
 # METHODS
 
 sub is_pattern {
-    my ( undef, $what ) = @_;
-    return exists $patterns{$what};
+    my ( undef, $pat ) = @_;
+    return exists $patterns{$pat};
 }
 
 sub render {
     my ($self) = @_;
 
-    my $sp = $self->pattern // die "subpattern not set";
-    confess "subpattern $sp does not exist" unless exists $patterns{$sp};
-
-    my $type = ref $patterns{$sp};
     my $ret;
+    my $target = $self->target;
+    my $type   = ref $target;
 
     # this complication allows for subpatterns to point at other parse
     # trees instead of just simple terminal strings (yes, you could
@@ -55,25 +55,17 @@ sub render {
     #
     # NOTE walk sub must be kept in sync with this logic
     if ( !$type ) {
-        $ret = $patterns{$sp};
+        $ret = $target;
     } else {
         if ( $type eq 'ARRAY' ) {
             # do not need Math::Random::Discrete here as the weights are
             # always equal; for weighted instead write that unit out
             # manually via [a*2/e/i/o/u] or such
-            $ret = @{ $patterns{$sp} }[ rand @{ $patterns{$sp} } ] // '';
-        } elsif ( $patterns{$sp}->can('render') ) {
-            $ret = $patterns{$sp}->render;
+            $ret = @{$target}[ rand @$target ] // '';
         } else {
-            # this will most likely be from a set_patterns call done
-            # somewhere else in the code; the backtrace should help find
-            # the first ->render call and then one can look before that
-            # for set_patterns (or possibly update_pattern) calls
-            confess "subpattern $sp points to unknown type";
+            $ret = $target->render;
         }
-        $ret = $ret->recurse if ref $ret;
     }
-
     return $ret;
 }
 
@@ -94,11 +86,10 @@ sub update_pattern {
     croak "update needs a pattern and a list of values\n" unless @_;
     croak "value must be defined" if !defined $_[0];
 
-    # NOTE that list-versus-a-single-arrayref are different in that a
-    # copy is made of the list, while the arrayref is passed as-is; this
-    # allows the caller to fiddle around with the arrayref without in
-    # turn needing to make new update_pattern calls for each change.
-    # this may or may not be a good idea
+    # NOTE arrayref as single argument is saved without making a copy of
+    # the contents; this will allow the caller to potentially change
+    # that ref and thus influence what is stored in patterns after this
+    # update_pattern call
     $patterns{$pattern} = @_ == 1 ? $_[0] : [@_];
 
     return $class_or_self;
@@ -107,23 +98,14 @@ sub update_pattern {
 sub walk {
     my ( $self, $callback ) = @_;
 
-    my $sp = $self->pattern // die "subpattern not set";
-    confess "subpattern $sp does not exist" unless exists $patterns{$sp};
-
     $callback->($self);
 
-    # NOTE this logic should be kept in sync with render sub
-    my $type = ref $patterns{$sp};
-    if ( ref $type ) {
-        if ( $type eq 'ARRAY' ) {
-            for my $term (@{ $patterns{$sp} }) {
-                $term->walk if ref $term;
-            }
-        } elsif ( $patterns{$sp}->can('walk') ) {
-            $patterns{$sp}->walk;
-        } else {
-            confess "unknown type in pattern $sp";
-        }
+    my $target = $self->target;
+    my $type   = ref $target;
+
+    # NOTE this logic must be kept in sync with render sub
+    if ( $type and $type ne 'ARRAY' ) {
+        $target->walk($callback);
     }
     return;
 }
@@ -141,6 +123,7 @@ Lingua::Awkwords::Subpattern - implements named subpatterns
   use Lingua::Awkwords;
   use Lingua::Awkwords::Subpattern;
 
+  # pick-one-of-these patterns (equal weights)
   Lingua::Awkwords::Subpattern->set_patterns(
       C => [qw/p t k s m n/],
       N => [qw/m n/],
@@ -150,16 +133,25 @@ Lingua::Awkwords::Subpattern - implements named subpatterns
   my $triphthong = Lingua::Awkwords->new( pattern => q{ VVV } );
   say $triphthong->render;
 
+  # patterns can also point to parse trees
+  Lingua::Awkwords::Subpattern->update_pattern(
+      T => $triphthong
+  );
+
+  my $tritriphthong = Lingua::Awkwords->new( pattern => q{ TTT } );
+  say $tritriphthong->render;
+
 =head1 DESCRIPTION
 
 Subpatterns are named (with the ASCII letters C<A-Z>) elements of an
-awkwords pattern that expand out to some list of choices equally
-weighted. That is, C<V> in a I<pattern> can be a shorthand notation for
+awkwords pattern that expand out to some list of equally weighted
+choices, or existing parse tree objects. That is, C<V> in a I<pattern>
+can be a shorthand notation for
 
   [a/e/i/o/u]
 
 See the source code for what patterns are defined by default, or use
-B<set_patterns> or B<update_pattern> first to change the values.
+B<set_patterns> or B<update_pattern> to change the values.
 
 =head1 ATTRIBUTES
 
@@ -171,6 +163,12 @@ The pattern this object represents. Mandatory. Typically should be an
 ASCII letter in the C<A-Z> range and typically should be set via the
 B<new> method.
 
+=item I<target>
+
+Array reference or object the I<pattern> points to. Prior to 0.03 this
+did not exist, and the target would be looked up from the patterns
+each time.
+
 =back
 
 =head1 METHODS
@@ -184,44 +182,44 @@ pattern or not.
 
 =item B<new>
 
-Constructor. A I<pattern> should ideally be supplied. Will blow up if
-the I<pattern> does not exist in the global patterns list.
+Constructor. A I<pattern> should ideally be supplied. Will throw an
+error if the I<pattern> does not exist in the global patterns list.
 
   Lingua::Awkwords::Subpattern->new( pattern => 'V' )
 
 =item B<render>
 
-Returns a random item from the list of choices for the I<pattern>
-that was hopefully set by some previous call. Blows up if I<pattern>
-was not set.
+Returns a random item from the list of choices, or calls B<render> if
+the I<target> is a reference of the not-ARRAY kind.
 
 =item B<set_patterns> I<list-of-patterns-and-choices>
 
-Resets I<all> the choices for multiple patterns. These changes are
-global to a process. For example for the Toki Pona language one might
-set C<C> for consonants and C<V> for vowels via
+Resets I<all> the choices. These changes are global to a process. For
+example for the Toki Pona language one might set C<C> for consonants and
+C<V> for vowels via
 
   Lingua::Awkwords::Subpattern->set_patterns(
       C => [qw/j k l m n p s t w/],
       V => [qw/a e i o u/],
   );
 
-Choices can either be simple string values or objects capable of having
-B<render> called on them. L<Lingua::Awkwords/COMPLICATIONS> has an
-example of the later form.
+Choices can either be simple string values or an object ideally with a
+B<render> method. L<Lingua::Awkwords/COMPLICATIONS> has an example of
+the later form.
 
 =item B<update_pattern> I<pattern> I<choices>
 
-Updates the choices for the given I<pattern>. This happens globally in a
-process; all instances will see the change in future B<render> calls.
+Updates the choices for the given I<pattern>. This happens globally in
+a process.
 
 Note that array references are treated differently than lists of values;
 
   my $nnmm = [qw/n m/];
   Lingua::Awkwords::Subpattern->update_pattern( N => $nnmm );
 
-allows the array reference C<$nnmm> to be changed by the caller (thus
-affecting future B<render> calls for that I<pattern>), while
+allows the array reference C<$nnmm> to be changed by the caller
+(thus affecting future I<pattern> of future objects created in this
+class), while
 
   Lingua::Awkwords::Subpattern->update_pattern( N => @$nnmm );
 
@@ -230,9 +228,8 @@ the list of choices has been made.
 
 =item B<walk> I<callback>
 
-Calls the I<callback> function with itself as the argument, then
-tries to find anything the I<pattern> points to that can have B<walk>
-called on it and calls that.
+Calls the I<callback> function with itself as the argument, then if the
+I<target> is an object calls B<walk> on that object.
 
 =back
 
@@ -252,7 +249,8 @@ L<https://github.com/thrig/Lingua-Awkwords>
 
 There can only be 26 named subpatterns and these are global to the
 process. It may be beneficial to (optionally?) make them instance
-specific somehow.
+specific somehow, though a workaround for that is to incrementally build
+up a complicated parse tree from one or more other parse trees.
 
 =head1 SEE ALSO
 
